package fr.jussieu.script;
import java.io.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

/**
 * La classe <code>Deug</code> permet de raliser:
 * <ul>
 * <li>des entres/sorties (lecture/criture clavier/cran ou sur fichier),
 * <li>l'excution de commandes <em>systmes</em>
 * <li>des dessins,
 * <li>quelques manipulations lmentaires de chanes de caractres.
 * </ul>
 * L'ide conductrice est de masquer certaines constructions de Java que
 * nous avons ( l'Universit Paris Diderot)
 * considres comme trop avances pour une initiation  la programmation.
 * De plus cela permet d'obtenir une certaine uniformit des incantations Java.
 *
 * <H3>Entres/Sorties</H3>
 * <P>Les entres ne prennent leur source qu'au clavier et les sorties
 * ne se ralisent qu' l'cran du terminal.
 * <P> Les fonctions
 * <CODE>print(<I>type</I>)</CODE> et
 * <CODE>println(<I>type</I>)</CODE> permettant d'afficher tout <I>type</I> de
 * donnes (la seconde srie produit en plus un retour  la ligne).
 * <P>Les entres se font par l'intermdiaire des fonctions
 * <CODE>read<I>Type</I>()</CODE> pour tout type de donnes primitives Java.
 * Chacune de ces fonctions positionne, en cas d'erreur, un tat que l'on peut
 * interroger par les fonctions <CODE>isOk()</CODE> et
 * <CODE>getStatus()</CODE>.
 * <H4>Les fichiers...</H4>
 * <P>On trouvera dans ce qui suit, un exemple d'utilisation des fonctions de lecture et d'criture :
 * <PRE>
 *	if (Deug.ouvrirFichierEnEcriture("toto.txt")==Deug.NO_ERROR) {
 *	    Deug.ecrireUneLigneDansLeFichier("voici une premire ligne");
 *	    Deug.ecrireUneLigneDansLeFichier("voici une seconde ligne");
 *	    Deug.ecrireUneLigneDansLeFichier("et la dernire");
 *	    Deug.fermerFichierEnEcriture();
 *	}
 *	if (Deug.ouvrirFichierEnLecture("toto.txt")==Deug.NO_ERROR) {
 *	    String s;
 *	    while ((s=Deug.lireUneLigneDuFichier())!=null) System.out.println(s);
 *	    Deug.fermerFichierEnLecture();
 *	}
 * </PRE>
 *
 * <H3>Excution externe</H3>
 * <P>Une interface permet d'excuter des commandes systmes et de lire les
 * rsultats produits sur l'cran du terminal. La fonction 
 *<CODE>exec(String <I>commande</I>)</CODE> permet de lancer l'excution de 
 * la commande et de ses arguments ne formant qu'une seule chane de caractre.
 * La fonction <CODE>exec(String []<I>args</I>)</CODE> permet de lancer une 
 * commande spcifie par la <EM>liste de ses argumets</EM>, donc le premier
 * tant la commande elle-mme, puis le second le premier argument, etc.
 * L'exemple suivant illustre une utilisation possible de la fonction :
 * <PRE>
 * Deug.exec("ls -ail /");
 * </PRE>
 * ou
 * <PRE>
 * String [] c = new String[3];
 * c[0] = new String("ls");
 * c[1] = new String("-ail");
 * c[2] = new String("/");
 * Deug.exec(c);
 * </PRE>
 *
 * <H3>Dessins</H3>
 *
 * <P>Un exemple de programme graphique typique est le suivant:
 * <PRE>
 * Deug.startDrawings(200,200);
 * for (i=0; i<200; i+=4) {
 *   Deug.setGray(i);
 *   Deug.drawLine(0,i,199,199-i);
 *   Deug.drawLine(i,0,199-i,199);
 * }
 * Deug.stopDrawings();
 * </PRE>
 * Pour obtenir:<BR>
 * <IMG SRC="Dessin.jpg">
 * <H3>Chanes</H3>
 * On trouvera les fonctions <I>type</I>ToString() et stringTo<I>type</I>()
 * o (<I>type</I> peut tre:
 * <CODE>boolean</CODE>, <CODE>int</CODE>, <CODE>long</CODE>,
 * <CODE>float</CODE> ou <CODE>double</CODE>) qui ralisent les conversions
 * attendues.
 * <P>
 * On trouvera aussi les fonctions de comparaison: <CODE>equals()</CODE>,
 * <CODE>compare()</CODE>,
 * <CODE>subString()</CODE>,
 * <CODE>charAt()</CODE> et
 * <CODE>length()</CODE>.
 *
 * @author Jean-Baptiste.Yunes@liafa.jussieu.fr
 * @author Fabien Tarissan (formerly @ pps.jussieu.fr)
 * @author Daniele.Varacca@pps.jussieu.fr
 * @version 0.21, 02/10/09
 */
public final class Deug {
    /**
     * A private internally useful class to support drawings...
     */
    private static class Drawable extends Canvas {
	private int width, height;
	private Image backingStore;
	private Graphics backingGraphics;
	public Drawable(int width,int height) {
	    setSize(width,height);
	}
	public void setSize(int width,int height) {
	    super.setSize(width,height);
	    this.width = width;
	    this.height = height;
	}
	public Dimension getPreferredSize() {
	    return new Dimension(width,height);
	}
	public void paint(Graphics g) {
	    if (backingStore==null) {
		backingStore = createImage(Drawable.this.width,Drawable.this.height);
		backingGraphics = backingStore.getGraphics();
	    }
	    g.drawImage(backingStore,0,0,this);
	}
	public void update(Graphics g) {
	    paint(g);
	}
	public void drawLine(int x0,int y0,int x1, int y1) {
	    if (backingGraphics==null) return;
	    backingGraphics.drawLine(x0,y0,x1,y1);
	    repaint();
	}
    
	public void drawRect(int x, int y, int l, int h) {
	    if (backingGraphics==null) return;
	    backingGraphics.drawRect(x,y,l,h);
	    repaint();
	}

	public void fillRect(int x, int y, int l, int h) {
	    if (backingGraphics==null) return;
	    backingGraphics.fillRect(x,y,l,h);
	    repaint();
	}

	public void drawOval(int x, int y, int l, int h) {
	    if (backingGraphics==null) return;
	    backingGraphics.drawOval(x,y,l,h);
	    repaint();
	}

	public void fillOval(int x, int y, int l, int h) {
	    if (backingGraphics==null) return;
	    backingGraphics.fillOval(x,y,l,h);
	    repaint();
	}

	public void drawArc(int x, int y, int l, int h, int angleInit, int angleCouvert) {
	    if (backingGraphics==null) return;
	    backingGraphics.drawArc(x,y,l,h, angleInit, angleCouvert);
	    repaint();
	}

	public void fillArc(int x, int y, int l, int h, int angleInit, int angleCouvert) {
	    if (backingGraphics==null) return;
	    backingGraphics.fillArc(x,y,l,h, angleInit, angleCouvert);
	    repaint();
	}

	public void setColor(int c1,int c2,int c3) {
	    if (backingGraphics==null) return;
	    backingGraphics.setColor(new Color(c1,c2,c3));
	}
	public void clearArea() {
	    if (backingGraphics==null) return;
	    backingGraphics.clearRect(0,0,width,height);
	    repaint();
	}
	public void sleep(int secondes) {
	    try {
		Thread.sleep(secondes*1000);
	    } catch(InterruptedException ie) {}
	}

    }

    /**
     * Rien  signaler en ce qui concerne la dernire opration
     * d'entre/sortie.
     */
    public static final int NO_ERROR=0;
    /**
     * La lecture sur le flot d'entre s'est rvle impossible.
     */
    public static final int STREAM_ERROR=1;
    /**
     * La conversion de type sur la chane de caractre lue sur l'entre
     * n'a pas t possible.
     */
    public static final int FORMAT_ERROR=2;
    /**
     * L'argument pass est invalide (ex: <CODE>==null</CODE>)
     **/
    public static final int ARG_ERROR=3;
    /**
     * Le fichier n'a pu tre ouvert.
     */
    public static final int OPEN_ERROR=4;
    /**
     * Une erreur a t rencontre lors d'une criture dans un fichier.
     */
    public static final int WRITE_ERROR=4;
    /**
     * Pas d'instanciation de la classe <code>Deug</code>
     */
    private Deug() { }
    /**
     * Rfrence sur le flot de dcoupage en lexmes.
     */
    private static MyTokenizer theTokenizer =  new MyTokenizer(new InputStreamReader(System.in));
    /**
     * Code d'erreur interne.
     */
    private static int error;
    /**
     * Renvoie l'tat de la dernire opration d'entre/sortie.
     * <DL>
     * <DT><CODE>true</CODE><DD>si tout s'est bien pass;
     * <DT><CODE>false</CODE><DD>si une erreur s'est produite. En ce cas, il
     * peut tre utile de consulter la cause de l'erreur
     * (<code>getStatus()</code>).
     * </DL>
     * Un appel est quivalent  <CODE>getStatus()==NO_ERROR</CODE>.
     * @return l'tat logique de la dernire opration.
     */
    public static boolean isOk() {
	return getStatus()==NO_ERROR;
    }
    /**
     * Renvoie l'erreur rencontre lors de la dernire opration
     * d'entre/sortie.
     * @return la dernire erreur recontre.
     */
    public static int getStatus() {
	return error;
    }

    /**
     * Lit sur l'entre un caractre et renvoie sa valeur.
     *
     * <P>Les octets lus l'ont t sur l'entre standard.
     *
     * @return le caractre lu sur l'entre. Si le caractre renvoy est
     * le caractre <code>&lt;nul&gt;</code> il est alors ncessaire de
     * consulter l'tat de l'opration (<code>isOk()</code>).
     */
    public static char readChar() {
	int c;
	try {
	    c = theTokenizer.read();
	    if (c!=MyTokenizer.EOF) {
		error = NO_ERROR;
		return (char)c;
	    } else {
		error = STREAM_ERROR;
		return '\u0000';
	    }
	} catch(java.io.IOException ioe) {
	    error = STREAM_ERROR;
	    return '\u0000';
	}
    }

    /**
     * Termine toute excution de la machine.
     */
    public static void exit() {
	System.exit(0);
    }

    /**
     * Lit l'ensemble des caractres jusqu' la prochaine fin de ligne.
     *
     * <P>Les caractres lus le sont par l'intermdiaire de la mthode
     * <CODE>readChar()</CODE>
     * @return la ligne suivante disponible sur l'entre.
     */
    public static String readLine() {
	StringBuffer theString=new StringBuffer();
	char c;
	while ( ((c=readChar())!='\n') && isOk() ) {
	    theString.append(c);
	}
	if (!isOk()) return null;
	return theString.toString();
    }

    /**
     * Convertit une chane de caractres en <CODE>boolean</CODE>.
     * @param s la chane
     * @return la valeur obtenue aprs conversion (<CODE>boolean</CODE>) <DL>
     * <DT><CODE>true</CODE><DD>si <CODE>s</CODE> est gale 
     * <CODE>"true"</CODE>
     * <DT><CODE>false</CODE><DD>sinon
     * </DL>
     **/
    public static boolean stringToBoolean(String s) {
	error = NO_ERROR;
	if (s==null) {
	    error = ARG_ERROR;
	    return false;
	}
        try {
            return Boolean.valueOf(s).booleanValue();
        } catch(NumberFormatException nfe) {
            error = FORMAT_ERROR;
            return false;
        }
    }
    /**
     * Convertit une chane de caractres en <CODE>int</CODE>.
     * <P>
     * En cas d'erreur de conversion l'tat est positionn en consquence.
     * @param s la chane  convertir
     * @return la valeur obtenue aprs conversion (<CODE>int</CODE>)
     * <B>Attention</B>: si la valeur renvoye est <CODE>0</CODE>, il est
     * ncessaire de consulter le statut de l'opration...
     **/
    public static int stringToInt(String s) {
	error = NO_ERROR;
	if (s==null) {
	    error = ARG_ERROR;
	    return (int)0;
	}
        try {
            return Integer.parseInt(s);
        } catch(NumberFormatException nfe) {
            error = FORMAT_ERROR;
            return (int)0;
        }
    }
    /**
     * Convertit une chane de caractres en <CODE>byte</CODE>.
     * <P>
     * En cas d'erreur de conversion l'tat est positionn en consquence.
     * @param s la chane  convertir
     * @return la valeur obtenue aprs conversion (<CODE>byte</CODE>)
     * <B>Attention</B>: si la valeur renvoye est <CODE>0</CODE>, il est
     * ncessaire de consulter le statut de l'opration...
     **/
    public static byte stringToByte(String s) {
	error = NO_ERROR;
	if (s==null) {
	    error = ARG_ERROR;
	    return (byte)0;
	}
        try {
            return Byte.parseByte(s);
        } catch(NumberFormatException nfe) {
            error = FORMAT_ERROR;
            return (byte)0;
        }
    }
    /**
     * Convertit une chane de caractres en <CODE>double</CODE>.
     * <P>
     * En cas d'erreur de conversion l'tat est positionn en consquence.
     * @param s la chane  convertir
     * @return la valeur obtenue aprs conversion (<CODE>double</CODE>)
     * <B>Attention</B>: si la valeur renvoye est <CODE>0</CODE>, il est
     * ncessaire de consulter le statut de l'opration...
     **/
    public static double stringToDouble(String s) {
	error = NO_ERROR;
	if (s==null) {
	    error = ARG_ERROR;
	    return (double)0;
	}
        try {
            return Double.parseDouble(s);
        } catch(NumberFormatException nfe) {
            error = FORMAT_ERROR;
            return (double)0;
        }
    }
    /**
     * Convertit une chane de caractres en <CODE>float</CODE>.
     * <P>
     * En cas d'erreur de conversion l'tat est positionn en consquence.
     * @param s la chane  convertir
     * @return la valeur obtenue aprs conversion (<CODE>float</CODE>)
     * <B>Attention</B>: si la valeur renvoye est <CODE>0</CODE>, il est
     * ncessaire de consulter le statut de l'opration...
     **/
    public static float stringToFloat(String s) {
	error = NO_ERROR;
	if (s==null) {
	    error = ARG_ERROR;
	    return (float)0;
	}
        try {
            return Float.parseFloat(s);
        } catch(NumberFormatException nfe) {
            error = FORMAT_ERROR;
            return (float)0;
        }
    }
    /**
     * Convertit une chane de caractres en <CODE>long</CODE>.
     * <P>
     * En cas d'erreur de conversion l'tat est positionn en consquence.
     * @param s la chane  convertir
     * @return la valeur obtenue aprs conversion (<CODE>long</CODE>)
     * <B>Attention</B>: si la valeur renvoye est <CODE>0</CODE>, il est
     * ncessaire de consulter le statut de l'opration...
     **/
    public static long stringToLong(String s) {
	error = NO_ERROR;
	if (s==null) {
	    error = ARG_ERROR;
	    return (long)0;
	}
        try {
            return Long.parseLong(s);
        } catch(NumberFormatException nfe) {
            error = FORMAT_ERROR;
            return (long)0;
        }
    }
    /**
     * Lit un lexme sur l'entre et le convertit en <CODE>int</CODE>.
     * <p>
     * En cas d'erreur (fin du flot ou problme de conversion) l'tat est
     * positionn en consquence.
     * @return le lexme suivant en tant qu'entier (<CODE>int</CODE>)
     */
    public static int readInt() {
        int tokenType;
	tokenType = theTokenizer.nextToken();
	if (tokenType==MyTokenizer.EOF) {
	    error = STREAM_ERROR;
	    return 0;
	}
	return stringToInt(theTokenizer.currentToken());
    }
    /**
     * Lit un lexme sur l'entre et le convertit en <CODE>long</CODE>.
     * <p>
     * En cas d'erreur (fin du flot ou problme de conversion) l'tat est
     * positionn en consquence.
     * @return le lexme suivant en tant qu'entier long (<CODE>long</CODE>)
     */
    public static long readLong() {
        int tokenType;
	tokenType = theTokenizer.nextToken();
	if (tokenType==MyTokenizer.EOF) {
	    error = STREAM_ERROR;
	    return 0;
	}
	return stringToLong(theTokenizer.currentToken());
    }
    /**
     * Lit un lexme sur l'entre et le convertit en <CODE>float</CODE>.
     * <p>
     * En cas d'erreur (fin du flot ou problme de conversion) l'tat est
     * positionn en consquence.
     * @return le lexme suivant en tant qu'entier (<CODE>float</CODE>)
     */
    public static float readFloat() {
        int tokenType;
	tokenType = theTokenizer.nextToken();
	if (tokenType==MyTokenizer.EOF) {
	    error = STREAM_ERROR;
	    return 0;
	}
	return stringToFloat(theTokenizer.currentToken());
    }
    /**
     * Lit un lexme sur l'entre et le convertit en <CODE>double</CODE>.
     * <p>
     * En cas d'erreur (fin du flot ou problme de conversion) l'tat est
     * positionn en consquence.
     * @return le lexme suivant en tant qu'entier (<CODE>double</CODE>)
     */
    public static double readDouble() {
        int tokenType;
	tokenType = theTokenizer.nextToken();
	if (tokenType==MyTokenizer.EOF) {
	    error = STREAM_ERROR;
	    return 0;
	}
	return stringToDouble(theTokenizer.currentToken());
    }
    /**
     * Lit un mot sur l'entre.
     * <p>
     * En cas d'erreur (fin du flot) l'tat est
     * positionn en consquence.
     * @return le mot suivant sur l'entre (<code>null</code> en cas d'chec).
     */
    public static String readString() {
        int tokenType;
	tokenType = theTokenizer.nextToken();
	if (tokenType==MyTokenizer.EOF) {
	    error = STREAM_ERROR;
	    return null;
	}
	error = NO_ERROR;
	return theTokenizer.currentToken();
    }
    /**
     * Affiche un caractre Unicode.
     * @param c Le caractre  afficher.
     */
    public static void print(char c) {
	System.out.print(c);
    }
    /**
     * Affiche la valeur d'un entier (<code>int</code>)
     * @param i L'entier  afficher
     */
    public static void print(int i) {
	System.out.print(i);
    }
    /**
     * Affiche la valeur d'un boolen (<code>boolean</code>)
     * @param b Le boolen  afficher
     */
    public static void print(boolean b) {
	System.out.print(b);
    }
    /**
     * Affiche la valeur d'un entier long (<code>long</code>)
     * @param l L'entier long  afficher
     */
    public static void print(long l) {
	System.out.print(l);
    }
    /**
     * Affiche la valeur d'un flottant simple prcision (<code>float</code>)
     * @param f Le flottant  afficher
     */
    public static void print(float f) {
	System.out.print(f);
    }
    /**
     * Affiche la valeur d'un flottant double prcision (<code>double</code>)
     * @param d Le flottant  afficher
     */
    public static void print(double d) {
	System.out.print(d);
    }
    /**
     * Affiche la reprsentation en <code>String</code> de l'objet
     * @param o L'objet  afficher
     */
    public static void print(Object o) {
	System.out.print(o);
    }
    /**
     * Affiche la chane de caractre (<code>String</code>)
     * @param s La chane  afficher
     */
    public static void print(String s) {
	System.out.print(s);
    }
    /**
     * Affiche un caractre suivi d'un saut de ligne.
     * Un appel  cette fonction est quivalent 
     * <code>print(c); println();</code>.
     * @param c Le caractre  afficher.
     */
    public static void println(char c) {
	System.out.println(c);
    }
    /**
     * Affiche la valeur d'un boolen (<code>boolean</code>) suivie
     * d'un saut de ligne.
     * Un appel  cette fonction est quivalent 
     * <code>print(b); println();</code>.
     * @param b Le boolen  afficher
     */
    public static void println(boolean b) {
	System.out.println(b);
    }
    /**
     * Affiche la valeur d'un entier (<code>int</code>) suivie
     * d'un saut de ligne.
     * Un appel  cette fonction est quivalent 
     * <code>print(i); println();</code>.
     * @param i L'entier  afficher
     */
    public static void println(int i) {
	System.out.println(i);
    }
    /**
     * Affiche la valeur d'un entier long (<code>long</code>) suivie
     * d'un saut de ligne.
     * Un appel  cette fonction est quivalent 
     * <code>print(l); println();</code>.
     * @param l L'entier long  afficher
     */
    public static void println(long l) {
	System.out.println(l);
    }
    /**
     * Affiche la valeur d'un flottant simple prcision (<code>float</code>)
     * suivie d'un saut de ligne.
     * Un appel  cette fonction est quivalent 
     * <code>print(f); println();</code>.
     * @param f Le flottant  afficher
     */
    public static void println(float f) {
	System.out.println(f);
    }
    /**
     * Affiche un flottant double prcision suivi d'une fin de ligne.
     * Un tel appel se comporte comme <code>print(d); println()</code>.
     * @param d Le <code>double</code>  afficher.
     */
    public static void println(double d) {
	System.out.println(d);
    }
    public static void println(Object o) {
	System.out.println(o);
    }
    public static void println(String s) {
	System.out.println(s);
    }
    /**
     * Affiche un saut de ligne.
     */
    public static void println() {
	System.out.println();
    }
    public static char charAt(String aString,int position) {
	return aString.charAt(position);
    }
    /**
     * Extrait une sous-chane d'une chane donne.
     * @param aString la chane d'o extraire
     * @param begin indice du premier caractre  extraire
     * @param end indice du caractre exclu de l'extraction
     * @return la chane extraite
     */
    public static String subString(String aString,int begin,int end) {
	return aString.substring(begin,end);
    }
    /**
     * Convertit un boolen en chane de caractres.
     * @param value le boolen
     * @return la chane
     */
    public static String booleanToString(boolean value) {
	return String.valueOf(value);
    }
    /**
     * Convertit un caractre en une de chane de caractres.
     * @param value le caractre
     * @return la chane
     */
    public static String charToString(char value) {
	return String.valueOf(value);
    }
    /**
     * Convertit un rel simple en chane de caractres.
     * @param value le rel simple prcision
     * @return la chane
     */
    public static String floatToString(float value) {
	return String.valueOf(value);
    }
    /**
     * Convertit un rel double en de chane de caractres.
     * @param value le rel double prcision
     * @return la chane
     */
    public static String doubleToString(double value) {
	return String.valueOf(value);
    }
    /**
     * Convertit un entier en chane de caractres.
     * @param value l'entier
     * @return la chane
     */
    public static String intToString(int value) {
	return String.valueOf(value);
    }
    /**
     * Convertit un entier long en chane de caractres.
     * @param value l'entier long
     * @return la chane
     */
    public static String longToString(long value) {
	return String.valueOf(value);
    }
    /**
     * Compare deux chanes de caractres.
     * @param aString Une chane de caractres.
     * @param anotherString Une chane de caractres.
     * @return <code>true</code> si les deux chanes sont gales,
     * <code>false</code> sinon.
     */
    public static boolean equals(String aString,String anotherString) {
	return aString.equals(anotherString);
    }
    /**
     * Compare deux chanes de caractres.
     * @param aString une premire chane
     * @param anotherString une seconde chane
     * @return 0 si les deux chanes sont gales; une valeur ngative
     * si la seconde est plus grande que la premire dans l'ordre
     * lexicographique et une valeur positive si la premire est plus grande
     * (dans l'ordre lexicographique) que la seconde.
     */
    public static int compare(String aString,String anotherString) {
	return aString.compareTo(anotherString);
    }
    /**
     * Calcule la longueur d'une chane de caractres.
     * @param aString la chane
     * @return la longueur
     */
    public static int length(String aString) {
	return aString.length();
    }
    private static JFrame theFrame = null;
    private static Drawable theDrawable = null;
    /**
     * Prpare une zone d'affichage de taille 300x300.
     */
    public static void startDrawings() {
	startDrawings(300,300);
    }
    private static class GraphicsInitializer implements Runnable {
	private int width, height;
	private JFrame theFrame;
	private Drawable theDrawable;
	public JFrame getTheFrame() {
	    return theFrame;
	}
	public Drawable getTheDrawable() {
	    return theDrawable;
	}
	public GraphicsInitializer(int w,int h) {
	    width = w;
	    height = h;
	}
	public void run() {
	    theFrame = new JFrame("Dessin");
	    theFrame.setResizable(false);
	    theDrawable = new Drawable(width,height);
	    theFrame.getContentPane().add(theDrawable);
	    theFrame.pack();
	    theFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	    theFrame.setVisible(true);
	}
    }
    /**
     * Prpare une zone d'affichage de taille arbitraire.
     * @param width La largeur de la zone d'affichage
     * @param height La hauteur de la zone d'affichage
     */
    public static void startDrawings(int width,int height) {
	if (theFrame!=null) return;
	GraphicsInitializer t = new GraphicsInitializer(width,height);
	try {
	    SwingUtilities.invokeAndWait(t);
	    theFrame = t.getTheFrame();
	    theDrawable = t.getTheDrawable();
	} catch(Exception e) {
	    System.err.println("Something's wrong setting up graphical context");
	}
    }
    /**
     * Ferme la fentre graphique.
     */
    public static void stopDrawings() {
	if (theFrame==null) return;
	theFrame.dispose();
	theFrame=null;
    }
    /**
     * Dessine un point dans la zone d'affichage.
     * @param x L'abscisse du point.
     * @param y L'ordonne du point.
     */
    public static void drawPoint(int x,int y) {
	if (theFrame==null)
	    throw new RuntimeException("il manque un appel  startDrawings()");
	theDrawable.drawLine(x,y,x,y);
    }
    /**
     * Modifie la couleur de l'encre des points  dessiner.
     * Les couleurs disponibles sont au nombre de 256 <em>niveaux</em> de gris,
     * de <code>0</code> pour coder le noir  <code>255</code> pour
     * coder le blanc. Un gris mdian s'obtient donc avec <code>127</code>.
     * @param c La couleur.
     */
    public static void setGray(int c) {
	if (theFrame==null)
	    throw new RuntimeException("il manque un appel  startDrawings()");
	theDrawable.setColor(c,c,c);
    }
    /**
     * Dessine un segment de droite dans la zone d'affichage.
     * @param x0 L'abscisse du premier point.
     * @param y0 L'ordonne du premier point.
     * @param x1 L'abscisse du second point.
     * @param y1 L'ordonne du second point.
     */
    public static void drawLine(int x0,int y0,int x1, int y1) {
	if (theFrame==null)
	    throw new RuntimeException("il manque un appel  startDrawings()");
	theDrawable.drawLine(x0,y0,x1,y1);
    }


    /**
     * Dessine un rectangle vide dans la zone d'affichage.
     * <p>
     * Les droites dlimitant le rectangle  gauche et  droite sont
     * situes aux abscisses <code>x</code> et <code>x+l</code>.
     * Celles dlimitant le rectangle en haut et en bas sont situes
     * aux ordonnes <code>y</code> et <code>y+h</code>.
     * @param x L'abscisse du coin nord-ouest du rectangle.
     * @param y L'ordonne du coin nord-ouest du rectangle.
     * @param l La largeur du rectangle.
     * @param h La hauteur du rectangle.
     */
    public static void drawRect(int x,int y,int l, int h) {
	if (theFrame==null)
	    throw new RuntimeException("il manque un appel  startDrawings()");
	theDrawable.drawRect(x,y,l,h);
    }

    /**
     * Dessine un rectangle plein dans la zone d'affichage.
     * <p>
     * Les droites dlimitant le rectangle  gauche et  droite sont
     * situes aux abscisses <code>x</code> et <code>x+l</code>.
     * Celles dlimitant le rectangle en haut et en bas sont situes
     * aux ordonnes <code>y</code> et <code>y+h</code>.
     * @param x L'abscisse du coin nord-ouest du rectangle.
     * @param y L'ordonne du coin nord-ouest du rectangle.
     * @param l La largeur du rectangle.
     * @param h La hauteur du rectangle.
     */
    public static void fillRect(int x,int y,int l, int h) {
	if (theFrame==null)
	    throw new RuntimeException("il manque un appel  startDrawings()");
	theDrawable.fillRect(x,y,l,h);
    }

    /**
     * Dessine un cercle vide dans la zone d'affichage.
     * @param x L'abscisse du centre du cercle.
     * @param y L'ordonne du centre du cercle.
     * @param r Le rayon du cercle.
     */
    public static void drawCircle(int x,int y,int r) {
	if (theFrame==null)
	    throw new RuntimeException("il manque un appel  startDrawings()");
	theDrawable.drawOval(x-r, y-r, 2*r, 2*r);
    }

    /**
     * Dessine un cercle plein dans la zone d'affichage.
     * @param x L'abscisse du centre du cercle.
     * @param y L'ordonne du centre du cercle.
     * @param r Le rayon du cercle.
     */
    public static void fillCircle(int x,int y,int r) {
	if (theFrame==null)
	    throw new RuntimeException("il manque un appel  startDrawings()");
	theDrawable.fillOval(x-r, y-r, 2*r, 2*r);
    }


    /**
     * Dessine un arc de cercle vide dans la zone d'affichage.
     * <p>
     * La valeur des angles est spcifie en degr.
     * Le sens de rotation est celui des cercles trigonomtriques.
     * L'angle de dpart est <code>angleInit</code>
     * et l'angle de fin <code>angleInit + angleCouvert</code>
     * @param x L'abscisse du centre du cercle.
     * @param y L'ordonne du centre du cercle.
     * @param r Le rayon du cercle.
     * @param angleInit Angle de dpart.
     * @param angleCouvert Angle de rotation.
     */
    public static void drawArc(int x,int y,int r, int angleInit, int angleCouvert) {
	if (theFrame==null)
	    throw new RuntimeException("il manque un appel  startDrawings()");
	theDrawable.drawArc(x-r, y-r, 2*r, 2*r, angleInit, angleCouvert);
    }

    /**
     * Dessine un arc de cercle plein dans la zone d'affichage.
     * <p>
     * La valeur des angles est spcifie en degr.
     * Le sens de rotation est celui des cercles trigonomtriques.
     * L'angle de dpart est <code>angleInit</code>
     * et l'angle de fin <code>angleInit + angleCouvert</code>
     * @param x L'abscisse du centre du cercle.
     * @param y L'ordonne du centre du cercle.
     * @param r Le rayon du cercle.
     * @param angleInit L'angle de dpart.
     * @param angleCouvert L'angle de rotation.
     */
    public static void fillArc(int x,int y,int r, int angleInit, int angleCouvert) {
	if (theFrame==null)
	    throw new RuntimeException("il manque un appel  startDrawings()");
	theDrawable.fillArc(x-r, y-r, 2*r, 2*r, angleInit, angleCouvert);
    }

    /**
     * Efface intgralement le contenu de la fentre graphique.
     */
    public static void clearArea() {
	if (theFrame==null)
	    throw new RuntimeException("il manque un appel  startDrawings()");
	theDrawable.clearArea();
    }

    private static void printExternalProcessOutputs(Process p) {
	try {
	    BufferedReader bf = new BufferedReader(new InputStreamReader(p.getInputStream()));
	    String s;
	    while ( (s=bf.readLine())!= null) {
		System.out.println(s);
	    }
	    bf.close();
	} catch (Exception ex) {
	    throw new RuntimeException("Impossible de lire les rsultats de la commande");
	}
    }

    /**
     * Excute une commande par l'intermdiaire d'un processus externe.
     * La commande est spcifie par son nom (premier argument du tableau) puis 
     * par ses options (arguments  partir du rang 2).
     * @param args Le tableau des arguments
     * @since 0.2
     */
    public static void exec(String []args) {
	Process p = null;
	try {
	    p = Runtime.getRuntime().exec(args);
	} catch(Exception ex) {
	    throw new RuntimeException("Impossible de lancer "+args[0]);
	}
	printExternalProcessOutputs(p);
	try {
	    p.waitFor();
	} catch (Exception ex) {}
    }

    /**
     * Excute une commande par l'intermdiaire d'un processus externe.
     * La commande est spcifie par une chane de caractre contenant son nom
     * et ses options. Les arguments devant tre spars par des espacements
     * (espace, tabulation).
     * @param command La chane de commande
     * @since 0.2
     */
    public static void exec(String command) {
	Process p = null;
	try {
	    p = Runtime.getRuntime().exec(command);
	} catch(Exception ex) {
	    throw new RuntimeException("Impossible de lancer "+command);
	}
	printExternalProcessOutputs(p);
	try {
	    p.waitFor();
	} catch (Exception ex) {}
    }

    private static BufferedReader bf;
    private static String rs;
    /**
     * Cette fonction permet de prparer un fichier de sorte que des lectures puissent y tre ralises.
     * @param nom Le nom du fichier  ouvrir, et qui doit exister pralablement.
     * @return <CODE>Deug.NO_ERROR</CODE> si tout s'est bien pass, <CODE>Deug.OPEN_ERROR</CODE> sinon.
     * @since 0.21
     * @see #fermerFichierEnLecture()
     * @see #lireUneLigneDuFichier()
     */
    public static int ouvrirFichierEnLecture(String nom) {
	if (bf!=null) throw new RuntimeException("il faut fermer le fichier dj ouvert avant d'en ouvrir un autre");
	try {
	    bf = new BufferedReader(new FileReader(nom));
	} catch(FileNotFoundException ex) {
	    return OPEN_ERROR;
	}
	return NO_ERROR;
    }
    /**
     * Cette fonction doit tre appelle lorsqu'on ne dsire plus effectuer de lecture dans le fichier
     * prcdemment ouvert.
     * @see #ouvrirFichierEnLecture(String nom)
     * @since 0.21
     */
    public static void fermerFichierEnLecture() {
	if (bf==null) return;
	try {
	    bf.close();
	} catch(IOException ex) {}
	bf = null;
    }
    /**
     * Cette fonction permet d'obtenir une ligne du fichier prcdemment ouvert.
     * La ligne est renvoye sous la forme d'une chane de caratres.
     * @return une chane de caractres; <CODE>null</CODE> s'il n'y a plus rien  lire.
     * @see #ouvrirFichierEnLecture(String nom)
     * @since 0.21
     */
    public static String lireUneLigneDuFichier() {
	if (bf==null)
	    throw new RuntimeException("il faut d'abord ouvrir un fichier avant de tenter de lire...");
	try {
	    return bf.readLine();
	} catch(IOException ex) {
	    return null;
	}
    }
    private static PrintWriter pw;
    /**
     * Cette fonction permet de prparer un fichier de sorte que des critures puissent y tre 
     * ralises.
     * @param nom Le nom du fichier  ouvrir, et qui doit exister pralablement.
     * @return <CODE>Deug.NO_ERROR</CODE> si tout s'est bien pass, <CODE>Deug.OPEN_ERROR</CODE> sinon.
     * @since 0.21
     * @see #fermerFichierEnEcriture()
     * @see #ecrireUneLigneDansLeFichier(String)
     */
    public static int ouvrirFichierEnEcriture(String nom) {
	if (pw!=null)
	    throw new RuntimeException("il faut fermer le fichier dj ouvert avant d'en ouvrir un autre");
	try {
	    pw = new PrintWriter(new FileWriter(nom));
	} catch(IOException ex) {
	    return OPEN_ERROR;
	}
	return NO_ERROR;
    }
    /**
     * Cette fonction doit tre appelle lorsqu'on ne dsire plus effectuer d'criture dans le fichier
     * prcdemment ouvert.
     * @see #ouvrirFichierEnEcriture(String nom)
     * @since 0.21
     */
    public static void fermerFichierEnEcriture() {
	if (pw==null) return;
	pw.close();
	pw = null;
    }
    /**
     * Cette fonction permet d'crire une ligne dans le fichier prcdemment ouvert.
     * La ligne crite est constitue de la chaine de caractres suivie d'un caractre de fin de ligne.
     * @return une chane de caractres; <CODE>null</CODE> s'il n'y a plus rien  lire.
     * @see #ouvrirFichierEnEcriture(String nom)
     * @since 0.21
     */
    public static int ecrireUneLigneDansLeFichier(String aEcrire) {
	if (pw==null)
	    throw new RuntimeException("il faut d'abord ouvrir un fichier avant de tenter d'y crire...");
	pw.println(aEcrire);
	return pw.checkError() ? WRITE_ERROR : NO_ERROR;
    }
}
